package org.zdevra.sparrow;

import com.google.inject.Provider;
import org.zdevra.sparrow.channel.MessageChannel;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * Provider is responsible for instantiating the channels in Guice.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
class ChannelProvider implements Provider<MessageChannel>
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** message channel class that we want to create */
	private final Class<? extends MessageChannel> channelClass;

	/** name for what we want to create channel */
	private final String name;


	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param channelClass
	 * @param name
	 */
	public ChannelProvider(Class<? extends MessageChannel> channelClass, String name)
	{
		this.channelClass = channelClass;
		this.name = name;
	}


	@Override
	public MessageChannel get()
	{
		try {
			Constructor<? extends MessageChannel> constructor = channelClass.getConstructor(null);
			Object instance = constructor.newInstance(null);

			if (instance instanceof Nameable) {
				((Nameable)instance).setName(name);
			}

			return (MessageChannel) instance;

		} catch (InvocationTargetException e) {
			throw new IllegalStateException("Error in Channel provider:" + e.getMessage(), e);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException("Error in Channel provider:" + e.getMessage(), e);
		} catch (InstantiationException e) {
			throw new IllegalStateException("Error in Channel provider:" + e.getMessage(), e);
		} catch (IllegalAccessException e) {
			throw new IllegalStateException("Error in Channel provider:" + e.getMessage(), e);
		}
	}
}
