/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

import java.io.Serializable;
import java.lang.annotation.Annotation;

/**
 * Implementation of {@link ChannelNamed}
 *
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
class ChannelNamedImpl implements ChannelNamed, Serializable
{
	//-------------------------------------------------------------------------------------------------------------------
	// members & constants
	//-------------------------------------------------------------------------------------------------------------------
	private static final long serialVersionUID = 0;

	/** channel's name */
	private final String value;

	//-------------------------------------------------------------------------------------------------------------------
	// constructors & methods
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * Constructor
	 * @param value
	 */
	public ChannelNamedImpl(String value) {
		this.value = (value != null) ? value : "unknown";
	}


	/**
	 * returns channel name
	 * @return
	 */
	public String value() {
		return this.value;
	}

	//-------------------------------------------------------------------------------------------------------------------
	// impl. methods
	//-------------------------------------------------------------------------------------------------------------------

	@Override
	public int hashCode() {
		return (127 * "value".hashCode()) ^ value.hashCode();
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof ChannelNamed)) {
			return false;
		}

		ChannelNamed other = (ChannelNamed) o;
		return value.equals(other.value());
	}

	@Override
	public String toString() {
		return "@" + ChannelNamed.class.getName() + "(value=" + value + ")";
	}

	@Override
	public Class<? extends Annotation> annotationType() {
		return ChannelNamed.class;
	}


}
